@@ -8,8 +8,7 @@ from ssht00ls.v1.classes import utils
 def __check_os__(supported=[]):
 	response = utils.__default_response__()
 	if OS not in supported:
-		response["error"] = "Unsupported operating system [{}].".format(OS)
-		return False,response
+		return False, r3sponse.error_response("Unsupported operating system [{}].".format(OS))
 	return True,response
 
 # the manager class.
@@ -18,9 +17,6 @@ class SmartCards(object):
 		a=1
 		#
 	def scan(self, silent=False):
-		
-		# initialize response.
-		response = utils.__default_response__()
 
 		# list.
 		output = utils.__execute__(["ykman", "list"], shell=False, return_format="array")
@@ -34,16 +30,30 @@ class SmartCards(object):
 					text = card.split(" Serial:")[0]
 					smart_cards[serial_number] = SmartCard(serial_number=serial_number)
 				except IndexError: 
-					response["error"] = "Unrecognized smart card detected. Remove the smart card and plug it back in."
-					return response
+					return r3sponse.error_response("Unrecognized smart card detected. Remove the smart card and plug it back in.")
 
 		# response.
-		response["success"] = True
-		response["message"] = f"Successfully scanned & detected {len(smart_cards)} smart card(s)."
-		response["smart_cards"] = smart_cards
-		return response
+		return r3sponse.success_response(f"Successfully scanned & detected {len(smart_cards)} smart card(s).", {
+			"smart_cards":smart_cards
+		})
 
 		#
+	def find_smart_card(self, serial_number=None):
+		
+		# scan for connected smart cards.
+		response = self.scan()
+		if response["error"] != None: return response
+		elif str(serial_number) not in list(response["smart_cards"].keys()):
+			return r3sponse.error_response(f"There is no smart card detected  with serial number [{serial_number}].")
+
+		# success.
+		smart_card = response["smart_cards"][str(serial_number)]
+		return r3sponse.success_response(f"Successfully initialzed smart card [{serial_number}].", {
+			"smart_card":smart_card
+		})
+
+		#
+	# system functions.
 	def __single_key_plugged_in__(self):
 
 		# scan.
@@ -53,18 +63,14 @@ class SmartCards(object):
 
 		# check one.
 		if len(l_response["smart_cards"]) > 1:
-			response["error"] = "There are multiple smart cards plugged in."
-			return response
+			return r3sponse.error_response("There are multiple smart cards plugged in.")
 
 		# check zero.
 		if len(l_response["smart_cards"]) == 0:
-			response["error"] = "There are no smart cards plugged in / detected."
-			return response
+			return r3sponse.error_response("There are no smart cards plugged in / detected.")
 
 		# success.
-		response["success"] = True
-		response["message"] = "There is only one smart cards plugged in."
-		return response
+		return r3sponse.success_response("There is only one smart cards plugged in.")
 
 # the smart card class.
 class SmartCard(object):
@@ -88,7 +94,7 @@ class SmartCard(object):
 				"/usr/lib/arm-linux-gnueabihf/opensc-pkcs11.so",
 				"/usr/local/lib/libykcs11",
 			]:
-				if os.path.exists(self.path):
+				if os.path.exists(path):
 					self.path = path
 					break
 			if self.path == None: raise ValueError("Unable to locate opensc-pkcs11.so path.")
@@ -96,6 +102,10 @@ class SmartCard(object):
 		elif OS in ["osx"]: 
 			self.path = "/usr/local/lib/libykcs11.dylib"
 
+		# check permissions.
+		#fp = Formats.FilePath(self.path)
+		#fp.permission.check(permission=600)
+
 		# variables.
 		self.puk = None
 		self.pin = None
@@ -113,8 +123,7 @@ class SmartCard(object):
 		try: 
 			output = subprocess.check_output(f"ykman --device {self.serial_number} piv info", shell=True).decode().replace('  ',' ').replace('  ',' ').replace('  ',' ').lower().split('\n')
 		except: 
-			response["error"] = "failed to retrieve yubikey info."
-			return response
+			return r3sponse.error_response("Failed to retrieve yubikey info.")
 
 		# iterate.
 		info = {}
@@ -128,9 +137,7 @@ class SmartCard(object):
 		response["info"] = info
 
 		# success.
-		response["success"] = True
-		response["message"] = f"Successfully retrieved the information from smart card [{self.serial_number}]."
-		return response
+		return r3sponse.success_response(f"Successfully retrieved the information from smart card [{self.serial_number}].")
 
 		#
 	def unblock_pin(self, 
@@ -161,13 +168,10 @@ class SmartCard(object):
 
 		# handle error.
 		if output != "":
-			response["error"] = f"Unknown error pin unblocking, output: [{output}]."
-			return response
+			return r3sponse.error_response(f"Unknown error pin unblocking, output: [{output}].")
 
 		# handle success.
-		response["message"] = f"Successfully unblocked the pin code of smart card [{self.serial_number}]."
-		response["success"] = True
-		return response
+		return r3sponse.success_response(f"Successfully unblocked the pin code of smart card [{self.serial_number}].")
 
 		#
 	def change_pin(self, 
@@ -199,14 +203,11 @@ class SmartCard(object):
 
 		# handle success.
 		elif "New PIN set." in output: 
-			response["message"] = f"Successfully changed the pin of smart card [{self.serial_number}]."
-			response["success"] = True
-			return response
+			return r3sponse.success_response(f"Successfully changed the pin of smart card [{self.serial_number}].")
 
 		# unknown error.
 		else:
-			response["error"] = f"Unknown error while changing pin, output: [{output}]."
-			return response
+			return r3sponse.error_response(f"Unknown error while changing pin, output: [{output}].")
 
 		#
 	def change_puk(self, 
@@ -239,14 +240,12 @@ class SmartCard(object):
 
 		# handle success.
 		elif "New PUK set." in output: 
-			response["message"] = f"Successfully changed the puk of smart card [{self.serial_number}]."
-			response["success"] = True
+			return r3sponse.success_response(f"Successfully changed the puk of smart card [{self.serial_number}].")
 			return response
 
 		# unknown error.
 		else:
-			response["error"] = f"Unknown error while changing puk, output: [{output}]."
-			return response
+			return r3sponse.error_response(f"Unknown error while changing puk, output: [{output}].")
 
 		#
 	def generate_key(self, 
@@ -273,8 +272,7 @@ class SmartCard(object):
 		success, response = self.__handle_default_output__(output)
 		if not success: return response
 		elif output != "":
-			response["error"] = f"Unknown error during key generation, output: [{output}]."
-			return response
+			return r3sponse.error_response(f"Unknown error during key generation, output: [{output}].")
 
 		# do.
 		command = f'ykman --device {self.serial_number} piv generate-certificate -s "/CN=SSH-key/" 9a public.pem --pin {pin} --management-key 010203040506070801020304050607080102030405060708'
@@ -284,13 +282,10 @@ class SmartCard(object):
 		success, response = self.__handle_default_output__(output)
 		if not success: return response
 		elif output != "":
-			response["error"] = f"Unknown error during certificate generation, output: [{output}]."
-			return response
+			return r3sponse.error_response(f"Unknown error during certificate generation, output: [{output}].")
 
 		# handle success.
-		response["message"] = f"Successfully generated a signed certificate & key for smart card [{self.serial_number}]."
-		response["success"] = True
-		return response
+		return r3sponse.success_response(f"Successfully generated a signed certificate & key for smart card [{self.serial_number}].")
 
 		#
 	def generate_management_key(self, 
@@ -317,12 +312,9 @@ class SmartCard(object):
 		success, response = self.__handle_default_output__(output)
 		if not success: return response
 		elif output != "":
-			response["error"] = f"Unknown error during management key generation, output: [{output}]."
-			return response
+			return r3sponse.error_response(f"Unknown error during management key generation, output: [{output}].")
 		else:
-			response["message"] = f"Successfully generated a management key for smart card [{self.serial_number}]."
-			response["success"] = True
-			return response
+			return r3sponse.success_response(f"Successfully generated a management key for smart card [{self.serial_number}].")
 
 		#
 	def reset_piv(self): # for when both pin & puk codes are blocked.
@@ -337,13 +329,11 @@ class SmartCard(object):
 		# handle success.
 		if "Success!" in output:
 			response["success"] = True
-			response["message"] = "Successfully resetted the smart card."
-			return response
+			return r3sponse.success_response("Successfully resetted the smart card.")
 
 		# handle error.
 		else:
-			response["error"] = "Failed to reset the smart card."
-			return response
+			return r3sponse.error_response("Failed to reset the smart card.")
 
 	# single key plugged in compatible:
 	def export_keys(self, 
@@ -358,8 +348,7 @@ class SmartCard(object):
 		
 		# error.
 		if len(output) == 0 or "ssh-rsa " not in output[0]:
-			response["error"] = f"Failed to export smart card [{self.serial_number}]."
-			return response
+			return r3sponse.error_response(f"Failed to export smart card [{self.serial_number}].")
 		else:
 
 			# write out.
@@ -367,14 +356,10 @@ class SmartCard(object):
 				try:
 					utils.__save_file__(path, utils.__array_to_string__(output, joiner="\n"))
 				except:
-					response["error"] = f"Failed to write out the exported key from smart card [{self.serial_number}]."
-					return response
+					return r3sponse.error_response(f"Failed to write out the exported key from smart card [{self.serial_number}].")
 
 			# success.
-			response["success"] = True
-			response["message"] = f"Successfully exported smart card [{self.serial_number}]."
-			response["public_keys"] = output
-			return response
+			return r3sponse.success_response(f"Successfully exported smart card [{self.serial_number}].", {"public_keys":output})
 
 		#
 	def check_smart_card(self):
@@ -384,20 +369,13 @@ class SmartCard(object):
 		try:
 			output = subprocess.check_output("yubico-piv-tool -aversion", shell=True).decode()
 		except:
-			response["message"] = "Failed to check for yubikey smart cards."
-			return response
+			return r3sponse.error_response("Failed to check for yubikey smart cards.")
 
 		# success.
 		if "Application version " in output: 
-			response["success"] = True
-			response["smart_card"] = True
-			response["message"] = "Yubikey smart card detected."
-			return response
+			return r3sponse.success_response("Yubikey smart card detected.", {"smart_card":True})
 		else: 
-			response["success"] = True
-			response["smart_card"] = False
-			response["message"] = "No yubikey smart card detected."
-			return response
+			return r3sponse.success_response("No yubikey smart card detected.", {"smart_card":False})
 
 		#
 	def convert_to_smart_card(self):
@@ -420,9 +398,7 @@ class SmartCard(object):
 		l_response = self.check_smart_card()
 		if l_response["error"] != None: return l_response
 		if l_response["smart_card"]:
-			response["success"] = True
-			response["message"] = "The plugged in yubikey is already a smart cards."
-			return response
+			return r3sponse.success_response("The plugged in yubikey is already a smart cards.")
 
 		# check os.
 		success, response = __check_os__(["linux"])
@@ -474,57 +450,56 @@ class SmartCard(object):
 		self.console.log("Plug the key back in.", self.indent_increaser+2)
 
 		"""
-	def install(self):
+	def install(self, 
+		# specify a new pin (optional).
+		pin=None, 
+		# specify a new puk (optional).
+		puk=None,
+	):
 
 		# initialize.
 		response = utils.__default_response__()
+		if pin == None: pin = utils.__generate_pincode__(characters=6)
+		if puk == None: puk = utils.__generate_pincode__(characters=8)
 		info = {
-			"pin":utils.__generate_pincode__(characters=6),
-			"puk":utils.__generate_pincode__(characters=8),
+			"pin":pin,
+			"puk":puk,
 			"public_key":None,
 		}
 
 		# check single key plugged in.
-		print("__single_key_plugged_in__")
 		l_response = smart_cards.__single_key_plugged_in__()
 		if l_response["error"] != None: return l_response
 
 		# reset piv.
-		print("reset_piv")
 		l_response = self.reset_piv()
 		if l_response["error"] != None: return l_response
 
 		# convert to smart card.
-		print("convert_to_smart_card")
 		l_response = self.convert_to_smart_card()
 		if l_response["error"] != None: return l_response
 
 		# convert to smart card.
-		print("change_pin")
 		l_response = self.change_pin(new=info["pin"], old=123456)
 		if l_response["error"] != None: return l_response
 
 		# convert to smart card.
-		print("change_puk")
 		l_response = self.change_puk(new=info["puk"], old=12345678)
 		if l_response["error"] != None: return l_response
 
 		# convert to smart card.
-		print("generate_key")
 		l_response = self.generate_key(pin=info["pin"])
 		if l_response["error"] != None: return l_response
 
 		# convert to smart card.
-		print("generate_management_key")
 		l_response = self.generate_management_key(pin=info["pin"])
 		if l_response["error"] != None: return l_response
 
 		# success.
-		response["success"] = True
-		response["message"] = f"Successfully installed smart card [{self.serial_number}]."
-		response["pin"] = info["pin"]
-		response["puk"] = info["puk"]
-		return response
+		return r3sponse.success_response(f"Successfully installed smart card [{self.serial_number}].", {
+			"pin":info["pin"],
+			"puk":info["puk"],
+		})
 
 		#
 
@@ -537,29 +512,26 @@ class SmartCard(object):
 
 		# handle.
 		if "Incorrect PUK" in output:
-			response["error"] = f"Provided an incorrect puk code."
-			return False, response
+			return False, r3sponse.error_response(f"Provided an incorrect puk code.")
 		elif "Incorrect PIN" in output:
 			info = self.get_info()
 			if info['success']: response["error"] = f"Provided an incorrect pin code, {info['pin_attempts']} attempts left."
 			else: response["error"] = f"Provided an incorrect pin code."
 			return False, response
 		elif "PUK is blocked" in output:
-			response["error"] = f"The puk code of smart card [{self.serial_number}] is blocked."
-			return False, response
+			return False, r3sponse.error_response(f"The puk code of smart card [{self.serial_number}] is blocked.")
 		elif "Error: " in output:
-			response["error"] = output.split("Error: ")[1].replace('.\n', '. ').replace('\n', '')
-			return False, response
+			return False, r3sponse.error_response(output.split("Error: ")[1].replace('.\n', '. ').replace('\n', ''))
 
 		# success.
 		return True, response
 
 
-"""
-
 # initialized classes.
 smart_cards = SmartCards()
 
+"""
+
 # scan for connected smart cards.
 response = smart_cards.scan()
 
