@@ -162,7 +162,7 @@ class Config(object):
 		if not success: return response
 		
 		# disable key.
-		path = f'{HOME}/.ssh/config'
+		path = f'{H}/.ssh/config'
 		utils.__set_file_path_permission__(path, permission=755)
 		try:
 			data = utils.__load_file__(path)
@@ -204,9 +204,173 @@ class Config(object):
 		response["success"] = True
 		response["message"] = f"Successfully created a ssh config for client [{username}@{server}] (host alias: {username}.{server})."
 		return response
+	def check_keys(self, username=None, public_keys=[]):
 
+		# make readable.
+		if username == None: username = OWNER
+		sudo = OWNER != username
+		sudo_option = ""
+		if sudo: sudo_option = "sudo "
+		ssh_dir = Formats.FilePath(f"{HOME_BASE}/{username}/.ssh/", check_existance=False)
+		auth_keys = Formats.FilePath(f"{HOME_BASE}/{username}/.ssh/authorized_keys", check_existance=False)
+		ssh_dir.permission.set(permission=770, sudo=sudo, silent=True) # silent for when non existant.
+		auth_keys.permission.set(permission=770, sudo=sudo, silent=True) # silent for when non existant.
 
-# the sshfs object class.
+		# get output.
+		command = []
+		if sudo: command += ["sudo"]
+		output = utils.__execute__(command + ["cat", f"{HOME_BASE}/{username}/.ssh/authorized_keys"], return_format="array")
+		
+		# handle ssh dir not installed.
+		try:
+			if "No such file or directory" in output[0]:
+				ssh_dir.check(
+					directory=True,
+					permission=770,
+					owner=username,
+					group="administrator",
+					sudo=sudo,)
+				auth_keys.check(
+					data="",
+					permission=770,
+					owner=username,
+					group="administrator",
+					sudo=sudo,)
+				output = []
+		except IndexError: a=1
+
+		# check if already present.
+		for key in public_keys:
+			if key.replace("\n", "") not in output:
+				auth_keys.permission.set(permission=600, sudo=sudo)
+				ssh_dir.permission.set(permission=700, sudo=sudo)
+				key = key.replace("\n", "")
+				return r3sponse.error_response(f'Public key [{key}] is not activated.')
+
+		# set correct permission.
+		auth_keys.permission.set(permission=600, sudo=sudo)
+		ssh_dir.permission.set(permission=700, sudo=sudo)
+
+		# success.
+		return r3sponse.success_response(f'Successfully confirmed that the specfied {len(public_keys)} public key(s) are activated.')
+
+	def enable_keys(self, username=None, public_keys=[]):
+
+		# make readable.
+		if username == None: username = OWNER
+		sudo = OWNER != username
+		sudo_option = ""
+		if sudo: sudo_option = "sudo "
+		ssh_dir = Formats.FilePath(f"{HOME_BASE}/{username}/.ssh/", check_existance=False)
+		auth_keys = Formats.FilePath(f"{HOME_BASE}/{username}/.ssh/authorized_keys", check_existance=False)
+		ssh_dir.permission.set(permission=770, sudo=sudo, silent=True) # silent for when non existant.
+		auth_keys.permission.set(permission=770, sudo=sudo, silent=True) # silent for when non existant.
+
+		# get output.
+		command = []
+		if sudo: command += ["sudo"]
+		output = utils.__execute__(command + ["cat", f"{HOME_BASE}/{username}/.ssh/authorized_keys"], return_format="array")
+		
+		# handle ssh dir not installed.
+		try:
+			if "No such file or directory" in output[0]:
+				ssh_dir.check(
+					directory=True,
+					permission=770,
+					owner=username,
+					group="administrator",
+					sudo=sudo,)
+				auth_keys.check(
+					data="",
+					permission=770,
+					owner=username,
+					group="administrator",
+					sudo=sudo,)
+				output = []
+		except IndexError: a=1
+
+		# check if already present.
+		for key in public_keys:
+			if key.replace("\n", "") not in output:
+				os.system(f"{sudo_option}echo '{key}' >> {HOME_BASE}/{username}/.ssh/authorized_keys")
+
+		# check if added.
+		output = utils.__execute__(command + ["cat", f"{HOME_BASE}/{username}/.ssh/authorized_keys"], return_format="array")
+		for key in public_keys:
+			if key.replace("\n", "") not in output:
+				auth_keys.permission.set(permission=600, sudo=sudo)
+				ssh_dir.permission.set(permission=700, sudo=sudo)
+				key = key.replace("\n", "")
+				return r3sponse.error_response(f'Failed to enable public key [{key}].')
+	
+		# set correct permission.
+		auth_keys.permission.set(permission=600, sudo=sudo)
+		ssh_dir.permission.set(permission=700, sudo=sudo)
+
+		# success.
+		return r3sponse.success_response(f'Successfully enabled {len(public_keys)} public key(s).')
+
+		#
+	def disable_keys(self, username=None, public_keys=[]):
+
+		# make readable.
+		if username == None: username = OWNER
+		sudo = OWNER != username
+		ssh_dir = Formats.FilePath(f"{HOME_BASE}/{username}/.ssh/", check_existance=False)
+		auth_keys = Formats.FilePath(f"{HOME_BASE}/{username}/.ssh/authorized_keys", check_existance=False)
+		ssh_dir.permission.set(permission=770, sudo=sudo, silent=True) # silent for when non existant.
+		auth_keys.permission.set(permission=770, sudo=sudo, silent=True) # silent for when non existant.
+
+		# get output.
+		command = []
+		if sudo: command += ["sudo"]
+		output = utils.__execute__(command + ["cat", f"{HOME_BASE}/{username}/.ssh/authorized_keys"], return_format="array")
+		
+		# handle ssh dir not installed.
+		try:
+			if "No such file or directory" in output[0]:
+				ssh_dir.check(
+					directory=True,
+					permission=770,
+					owner=username,
+					group="administrator",
+					sudo=sudo,)
+				auth_keys.check(
+					data="",
+					permission=770,
+					owner=username,
+					group="administrator",
+					sudo=sudo,)
+				output = []
+		except IndexError: a=1
+
+		# check if already present.
+		new_keys = ""
+		for key in output:
+			if key.replace("\n", "") not in public_keys:
+				if new_keys == "":
+					new_keys = key+"\n"
+				else:
+					new_keys += key+"\n"
+		utils.__save_file__(f"{HOME_BASE}/{username}/.ssh/authorized_keys", new_keys)
+
+		# check if deleted.
+		output = utils.__execute__(command + ["cat", f"{HOME_BASE}/{username}/.ssh/authorized_keys"], return_format="array")
+		for key in public_keys:
+			if key.replace("\n", "") in output:
+				auth_keys.permission.set(permission=600, sudo=sudo)
+				ssh_dir.permission.set(permission=700, sudo=sudo)
+				key = key.replace("\n", "")
+				return r3sponse.error_response(f'Failed to disable public key [{key}].')
+
+		# set correct permission.
+		auth_keys.permission.set(permission=600, sudo=sudo)
+		ssh_dir.permission.set(permission=700, sudo=sudo)
+
+		# success.
+		return r3sponse.success_response(f'Successfully disabled {len(public_keys)} public key(s).')
+
+# the ssh key object class.
 class Key(object):
 	def __init__(self):
 
@@ -721,11 +885,22 @@ class Agent(object):
 				response["error"] = f"Key path [{path}] does not exist."
 				return response
 
+
 		# with passphrase.
+		response = utils.__default_response__()
 		if smart_card or passphrase != None:
 			self.initialize()
+			l_response = self.list()
+			if l_response["error"] != None: return l_response
+			count = len(l_response["keys"])
+			print("count 1:",count)
 			if smart_card:
-				spawn = pexpect.spawn(f'ssh-add -K {path}')
+				if OS in ["osx"]:
+					copy_path = "/usr/local/lib/libykcs11_NOTALINK.dylib" # working version 1
+					os.system(f"rm -fr {copy_path}")
+					os.system(f"cp {path} {copy_path}")
+					path = copy_path
+				spawn = pexpect.spawn(f'ssh-add -s {path}')
 			else:
 				spawn = pexpect.spawn(f'ssh-add {path}')
 
@@ -735,10 +910,10 @@ class Agent(object):
 				# handle pincode.
 				if smart_card:
 					spawn.expect(
-						f'Enter PIN for authenticator:',
+						f'Enter passphrase for PKCS#11:',
 						timeout=timeout,
 					)
-					spawn.sendline(str(pincode))
+					spawn.sendline(str(pin))
 				
 				# handle pass.
 				else:
@@ -768,10 +943,17 @@ class Agent(object):
 				response["error"] = f"Failed to add key [{path}] to the ssh agent #3."
 				return response
 
-			# success.
-			response["success"] = True
-			response["message"] = f"Successfully added key [{path}] to the ssh agent."
-			return response
+			# check success.
+			l_response = self.list()
+			if l_response["error"] != None: return l_response
+			print("count 2:",len(l_response["keys"]))
+			if count != len(l_response["keys"]):
+				response["success"] = True
+				response["message"] = f"Successfully added key [{path}] to the ssh agent."
+				return response
+			else: 
+				response["error"] = f"Failed to add key [{path}] to the ssh agent."
+				return response
 
 		# without pass.
 		else:
@@ -820,7 +1002,7 @@ class Agent(object):
 		if "Failed to communicate" in output:
 			response["error"] = "Failed to communicate with the ssh-agent."
 			return response
-		elif output == "The agent has no identities":
+		elif "The agent has no identities." in output:
 			response["keys"] = []
 		else:
 			response["keys"] = output
